@@ -448,15 +448,13 @@ class ha_rocksdb : public my_core::handler {
     }
   }
 
-  /** @brief
-    The name that will be used for display purposes.
-   */
-  const char *table_type() const /*override*/ {
-    DBUG_ENTER_FUNC();
-   // MariaDB: this function is not virtual, however ha_innodb
-   // declares it (and then never uses!) psergey-merge-todo:.
-    DBUG_RETURN(rocksdb_hton_name);
-  }
+  /*
+    MariaDB: this function:
+
+    const char *table_type() const
+
+    is non-virtual in class handler, so there's no point to override it.
+  */
 
   /* The following is only used by SHOW KEYS: */
   const char *index_type(uint inx) override {
